--[[ =================================================================
    Version 1.0
	
	Release:
		March 25, 2016
	
	Author:
		Zurom-Firetree
	
	Description:
        Gold Tracker Plus is a tool that provides extensive gold
		tracking capabilities. Allowing users to track gold across
		multiple categories and timelines, Gold Tracker Plus provides
		the most indepth gold accumulation data than any other addon!

    Contact:
        For questions, bug reports visit the website or send an email
        to the following address: fleury24@gmail.com

    Dependencies:
        None

    Credits:
        A big 'Thank You' to all the people at Blizzard Entertainment
        for making World of Warcraft.
    ================================================================= --]]

	-- System Vars
local GT_AddonName,GTP = ...;
local GT_Version = GetAddOnMetadata(GT_AddonName, "Version");
local L = GTP.L;

local isDebug = {
	merchant = false,
	timeline = false,
	quest = false,
	world = false,
	instance = false
};

-- String Vars
local plName,curMonth,curDay,curYear,trackType;

-- Integer Vars
local weekday,month,day,year;
local curMoney,newMoney;
local newGold,newSilver,newCopper;
local newWeekday,newMonth,newWeek,newYear;


-- Boolean Vars
local isTracking,isQuest,isVendor,isPurchase,isRepair;
local isWeekA,isWeekB,weekChange;

-- Merchant Vars
local SpendTotal,GainTotal,RepairTotal;

-- Mail vars
local mailVars = {
	isMail = false,
	isPostage = false,
	isGoldCheck = false,
	mailType = {
		receiving = false,
		sending = false,
	},
	isSendMoney = false,
	isCOD = false,
	numItems = 0,
	mailName = '',
	postage = 0,
	money = {
		gold = 0,
		silver = 0,
		copper = 0,
	}
}

local codName = '';

local WorldDayFrameText,WorldWeekFrameText,WorldMonthFrameText,WorldYearFrameText;
local InstanceDayFrameText,InstanceWeekFrameText,InstanceMonthFrameText,InstanceYearFrameText;
local QuestDayFrameText, QuestWeekFrameText,QuestMonthFrameText,QuestYearFrameText;
local VendorDayFrameText,VendorWeekFrameText,VendorMonthFrameText,VendorYearFrameText;
local SpentDayFrameText,SpentWeekFrameText,SpentMonthFrameText,SpentYearFrameText;
local RepairDayFrameText,RepairWeekFrameText,epairMonthFrameText,RepairYearFrameText;
	
local AllTimeWorldFrameText,AllTimeInstanceFrameText,AllTimeQuestFrameText,AllTimeVendorFrameText,AllTimeSpentFrameText,AllTimeRepairFrameText;
local AllTimeIncomeTotalFrameText,AllTimeExpenseTotalFrameText;
	
GTP.Events = CreateFrame("Frame");
GTP.GoldDialog = CreateFrame("Frame","GoldTrackerDialog",UIParent);
GTP.ArchiveDialog = CreateFrame("Frame","ArchiveDialog",UIParent);
local GoldDialog = GTP.GoldDialog;
local ArchiveDialog = GTP.ArchiveDialog;
local Events = GTP.Events;

local iCtr = 0;
local itr = itr or {};
local formatter = formatter or {};

Events:RegisterEvent("ADDON_LOADED");
Events:RegisterEvent("PLAYER_LOGIN");
Events:RegisterEvent("PLAYER_LOGOUT");
Events:RegisterEvent("PLAYER_ENTERING_WORLD");
Events:RegisterEvent("PLAYER_ALIVE");
Events:RegisterEvent("LOOT_OPENED");
Events:RegisterEvent("CHAT_MSG_MONEY");
Events:RegisterEvent("QUEST_COMPLETE");
Events:RegisterEvent("MERCHANT_SHOW");
Events:RegisterEvent("MAIL_SHOW");
Events:RegisterEvent("CHAT_MSG_SYSTEM");

-- Table Vars
local weekdays = weekdays or {};
local months = months or {};

local GT_OptionDropDownMenu = CreateFrame("Frame", "GT_OptionDropDownMenu");
GT_OptionDropDownMenu.displayMode = "MENU";
GT_OptionDropDownMenu.info = {};
GT_OptionDropDownMenu.UncheckHack = function(dropdownbutton)
	_G[dropdownbutton:GetName().."Check"]:Hide();
end
GT_OptionDropDownMenu.HideMenu = function()
	if UIDROPDOWNMENU_OPEN_MENU == GT_OptionDropDownMenu then
		CloseDropDownMenus();
	end
end
GTP.Archives = CreateFrame("Button","ArchiveButton",GoldDialog);
	GTP.Archives:SetHeight(20);
	GTP.Archives:SetWidth(75);
	GTP.Archives:SetPoint("TOPRIGHT",-25,-5);
	GTP.Archives:SetFrameStrata("FULLSCREEN");
	GTP.Archives:Show();
	
	-- ARCHIVE BUTTON LABEL
	local ArchiveButtonLabel = CreateFrame("Frame",nil,GTP.Archives);
	ArchiveButtonLabel:SetPoint("TOP",0,-2);
	ArchiveButtonLabel:SetWidth(50);
	ArchiveButtonLabel:SetHeight(20);
	ArchiveButtonLabel:SetFrameStrata("MEDIUM");
	GTP.ArchiveButtonLabel = ArchiveButtonLabel:CreateFontString(nil, "BACKGROUND", "GameFontNormal");
	local ArchiveButtonLabelText = GTP.ArchiveButtonLabel
	ArchiveButtonLabelText:SetPoint("TOP",0,-2);
	ArchiveButtonLabelText:SetFont("Fonts\\FRIZQT__.TTF", 11);
	ArchiveButtonLabelText:SetTextColor(1,1,0,1);
	ArchiveButtonLabelText:SetJustifyH("LEFT");
	ArchiveButtonLabelText:SetText(L["Archives"]);
	
GTP.Archives:SetScript("OnClick",function(self, button,down)
	if (button == "LeftButton") then
		ToggleDropDownMenu(1,nil,GT_OptionDropDownMenu,self:GetName(),0,0);
	end
end);
GTP.Archives:RegisterForClicks("LeftButtonUp");

local function DebugPrint(msg,debugger,...)
	if (isDebug[tostring(debugger)]) then
		print(msg);
	end
end

local function InitializeVars()
	table.insert(itr,tostring(iCtr)..". InitializeVars()");
	
	--Initialize Variables
	weekday,month,day,year = CalendarGetDate();
	table.insert(itr,"----- Day: '"..day.."'");
	table.insert(itr,"----- Weekday: '"..weekday.."'");
	table.insert(itr,"----- Month: '"..month.."'");
	table.insert(itr,"----- Year: '"..year.."'");
	plName = GetUnitName("player");
	
	iCtr = (iCtr or 0) + 1;
	
	weekdays[1] = L["Sunday"];
	weekdays[2] = L["Monday"];
	weekdays[3] = L["Tuesday"];
	weekdays[4] = L["Wednesday"];
	weekdays[5] = L["Thursday"];
	weekdays[6] = L["Friday"];
	weekdays[7] = L["Saturday"];
	
	months[1] = FULLDATE_MONTH_JANUARY;
	months[2] = FULLDATE_MONTH_FEBRUARY;
	months[3] = FULLDATE_MONTH_MARCH;
	months[4] = FULLDATE_MONTH_APRIL;
	months[5] = FULLDATE_MONTH_MAY;
	months[6] = FULLDATE_MONTH_JUNE;
	months[7] = FULLDATE_MONTH_JULY;
	months[8] = FULLDATE_MONTH_AUGUST;
	months[9] = FULLDATE_MONTH_SEPTEMBER;
	months[10] = FULLDATE_MONTH_OCTOBER;
	months[11] = FULLDATE_MONTH_NOVEMBER;
	months[12] = FULLDATE_MONTH_DECEMBER;
end

local function FormatArchiveValues(self,val)
	if (val == 0) then
		val = "|cFF8888880|r"
	else
		if (val < 100) then
			val = ("|cFFFF6600%dc |r"):format(val % 100);
		elseif (val < 10000) then
			val = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((val / 100) % 100, val % 100)
		else
			val = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(val / 100 / 100, (val / 100) % 100, val % 100)
		end
	end
	
	self:SetText(val);
end

local function ShowArchives(ListYear,ListMonth)
	ArchiveDialog:Show();
	if (not ListMonth) then
		GTP.ArchiveTitleLabel:SetText(tostring(ListYear));
		
		FormatArchiveValues(GTP.ArchiveWorldFrame,GTP_db[plName].archive[tostring(ListYear)].world);
		FormatArchiveValues(GTP.ArchiveInstanceFrame,GTP_db[plName].archive[tostring(ListYear)].instance);
		FormatArchiveValues(GTP.ArchiveQuestFrame,GTP_db[plName].archive[tostring(ListYear)].quest);
		FormatArchiveValues(GTP.ArchiveVendorFrame,GTP_db[plName].archive[tostring(ListYear)].vendor);
		FormatArchiveValues(GTP.ArchiveSpentFrame,GTP_db[plName].archive[tostring(ListYear)].spent);
		FormatArchiveValues(GTP.ArchiveRepairFrame,GTP_db[plName].archive[tostring(ListYear)].repair);
	else
		GTP.ArchiveTitleLabel:SetText(tostring(ListMonth).." "..tostring(ListYear));
	
		FormatArchiveValues(GTP.ArchiveWorldFrame,GTP_db[plName].archive[tostring(ListYear)][tostring(ListMonth)].world);
		FormatArchiveValues(GTP.ArchiveInstanceFrame,GTP_db[plName].archive[tostring(ListYear)][tostring(ListMonth)].instance);
		FormatArchiveValues(GTP.ArchiveQuestFrame,GTP_db[plName].archive[tostring(ListYear)][tostring(ListMonth)].quest);
		FormatArchiveValues(GTP.ArchiveVendorFrame,GTP_db[plName].archive[tostring(ListYear)][tostring(ListMonth)].vendor);
		FormatArchiveValues(GTP.ArchiveSpentFrame,GTP_db[plName].archive[tostring(ListYear)][tostring(ListMonth)].spent);
		FormatArchiveValues(GTP.ArchiveRepairFrame,GTP_db[plName].archive[tostring(ListYear)][tostring(ListMonth)].repair);
	end

end

GT_OptionDropDownMenu.initialize = function(self,level)
	local isManyYears;
	local numYears = 1;
	local numMenuItems = 2;
	local ListYears = GTP_db[plName].archive["years"];
	local SortedYears = ShortedYears or {};
	local numColon = string.find(ListYears,";");
	if (numColon) then
		-- Extract the existing years when there are more than one year in the archives
		ListYears = { strsplit(";",ListYears) };
		numYears = table.getn(ListYears);
		table.sort(ListYears);
		isManyYears = true;
	end
	
	if (not level) then
		return;
	end
	local info = self.info;
	wipe(info);
	
	if (level == 1) then
		-- Create the title of the menu
		info.isTitle = 1;
		info.text = L["Archives"];
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info,level);
		
		info.keepShownOnClick = 1;
		info.disabled = nil;
		info.isTitle = nil;
		info.notCheckable = nil;
		info.checked = nil;
		
		if (isManyYears) then
			-- List years when there are more than one in archives
			for i=1,numYears do
				info.text = ListYears[i];
				info.hasArrow = 1;
				info.value = "submenu"..ListYears[i];
				info.func = function(self) 
					_G[self:GetName().."Check"]:Hide();
					ShowArchives(ListYears[i],nil)
				end;
				UIDropDownMenu_AddButton(info,level);
				_G["DropDownList1Button"..(i+1).."UnCheck"]:Hide();
				numMenuItems = numMenuItems + 1;
			end
		else
			-- List the current year
			info.text = ListYears;
			info.func = function(self)
				_G[self:GetName().."Check"]:Hide();
				ShowArchives(ListYears,nil)
			end;
			info.hasArrow = 1;
			info.value = "submenu"..ListYears;
			UIDropDownMenu_AddButton(info,level);
			DropDownList1Button2UnCheck:Hide();
			numMenuItems = 3;
		end
		
		-- Close Menu Item
		info.hasArrow = nil;
		info.value = nil;
		info.notCheckable = nil;
		info.text = CLOSE;
		info.func = self.HideMenu;
		UIDropDownMenu_AddButton(info,level);
		_G["DropDownList1Button"..numMenuItems.."UnCheck"]:Hide();
	elseif (level == 2) then
		if (isManyYears) then
			-- List the months that exist in the archives within each of the multiple years
			local yearCtr = 1;
			for i=1,numYears do
				if (UIDROPDOWNMENU_MENU_VALUE == "submenu"..ListYears[i]) then
					for j=1,12 do
						if (GTP_db[plName].archive[tostring(ListYears[i])][tostring(months[j])]) then
							info.text = months[j];
							info.keepShownOnClick = 1;
							info.func = function(self)
								_G[self:GetName().."Check"]:Hide();
								ShowArchives(ListYears[i],months[j])
							end;
							UIDropDownMenu_AddButton(info,level);
							_G["DropDownList2Button"..yearCtr.."UnCheck"]:Hide();
							yearCtr = yearCtr + 1;
						end
					end
				end
			end
		else
			-- List the existing months for the current year
			if (UIDROPDOWNMENU_MENU_VALUE == "submenu"..ListYears) then
				for j=1,12 do
					local uMonth = string.gsub(" "..months[j], "%W%l", string.upper):sub(2);
					if (GTP_db[plName].archive[tostring(ListYears)][tostring(months[j])]) then
						info.text = L[uMonth];
						info.keepShownOnClick = 1;
						info.func = function(self)
							_G[self:GetName().."Check"]:Hide();
							ShowArchives(ListYears,months[j])
						end;
						UIDropDownMenu_AddButton(info,level);
						_G["DropDownList2Button1UnCheck"]:Hide();
					end
				end
			end
		end
	end
end;

local function FormatAllTimeValues(world,instance,quest,vendor,spent,repair)
	local income = world + instance + quest + vendor;
	local expense = spent + repair;
	
	if (world == 0) then
		world = "|cFF8888880|r"
	else
		if (world < 100) then
			world = ("|cFFFF6600%dc |r"):format(world % 100);
		elseif (world < 10000) then
			world = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((world / 100) % 100, world % 100)
		else
			world = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(world / 100 / 100, (world / 100) % 100, world % 100)
		end
	end
	
	if (instance == 0) then
		instance = "|cFF8888880|r"
	else
		if (instance < 100) then
			instance = ("|cFFFF6600%dc |r"):format(instance % 100);
		elseif (instance < 10000) then
			instance = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((instance / 100) % 100, instance % 100)
		else
			instance = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(instance / 100 / 100, (instance / 100) % 100, instance % 100)
		end
	end
	
	if (quest == 0) then
		quest = "|cFF8888880|r"
	else
		if (quest < 100) then
			quest = ("|cFFFF6600%dc |r"):format(quest % 100);
		elseif (quest < 10000) then
			quest = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((quest / 100) % 100, quest % 100)
		else
			quest = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(quest / 100 / 100, (quest / 100) % 100, quest % 100)
		end
	end
	
	if (vendor == 0) then
		vendor = "|cFF8888880|r"
	else
		if (vendor < 100) then
			vendor = ("|cFFFF6600%dc |r"):format(vendor % 100);
		elseif (vendor < 10000) then
			vendor = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((vendor / 100) % 100, vendor % 100)
		else
			vendor = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(vendor / 100 / 100, (vendor / 100) % 100, vendor % 100)
		end
	end
	
	if (spent == 0) then
		spent = "|cFF8888880|r"
	else
		if (spent < 100) then
			spent = ("|cFFFF6600%dc |r"):format(spent % 100);
		elseif (spent < 10000) then
			spent = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((spent / 100) % 100, spent % 100)
		else
			spent = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(spent / 100 / 100, (spent / 100) % 100, spent % 100)
		end
	end
	
	if (repair == 0) then
		repair = "|cFF8888880|r"
	else
		if (repair < 100) then
			repair = ("|cFFFF6600%dc |r"):format(repair % 100);
		elseif (repair < 10000) then
			repair = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((repair / 100) % 100, repair % 100)
		else
			repair = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(repair / 100 / 100, (repair / 100) % 100, repair % 100)
		end
	end
	
	if (income == 0) then
		income = "|cFF8888880|r"
	else
		if (income < 100) then
			income = ("|cFFFF6600%dc |r"):format(income % 100);
		elseif (income < 10000) then
			income = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((income / 100) % 100, income % 100)
		else
			income = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(income / 100 / 100, (income / 100) % 100, income % 100)
		end
	end
	
	if (expense == 0) then
		expense = "|cFF8888880|r"
	else
		if (expense < 100) then
			expense = ("|cFFFF6600%dc |r"):format(expense % 100);
		elseif (expense < 10000) then
			expense = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((expense / 100) % 100, expense % 100)
		else
			expense = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(expense / 100 / 100, (expense / 100) % 100, expense % 100)
		end
	end
	
	GTP.AT_W_Frame:SetText(world);
	GTP.AT_I_Frame:SetText(instance);
	GTP.AT_Q_Frame:SetText(quest);
	GTP.AT_V_Frame:SetText(vendor);
	GTP.AT_S_Frame:SetText(spent);
	GTP.AT_R_Frame:SetText(repair);
	
	GTP.AT_In_Frame:SetText(income);
	GTP.AT_Ex_Frame:SetText(expense);
end

local function FormatValues(dayVal,weekVal,monthVal,yearVal,totalVal,method)
	table.insert(formatter,"Method: '"..method.."', DayVal: '"..tostring(dayVal).."', WeekVal: '"..tostring(weekVal).."', MonthVal: '"..tostring(monthVal).."', YearVal: '"..tostring(yearVal).."'");
	if (dayVal == 0) then
		dayVal = "|cFF8888880|r";
	else
		if (dayVal < 100) then
			dayVal = ("|cFFFF6600%dc |r"):format(dayVal % 100);
		elseif (dayVal < 10000) then
			dayVal = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((dayVal / 100) % 100, dayVal % 100)
		else
			dayVal = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(dayVal / 100 / 100, (dayVal / 100) % 100, dayVal % 100)
		end
	end
	
	if (weekVal == 0) then
		weekVal = "|cFF8888880|r";
	else
		if (weekVal < 100) then
			weekVal = ("|cFFFF6600%dc |r"):format(weekVal % 100);
		elseif (weekVal < 10000) then
			weekVal = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((weekVal / 100) % 100, weekVal % 100)
		else
			weekVal = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(weekVal / 100 / 100, (weekVal / 100) % 100, weekVal % 100)
		end
	end
	
	if (monthVal == 0) then
		monthVal = "|cFF8888880|r";
	else
		if (monthVal < 100) then
			monthVal = ("|cFFFF6600%dc |r"):format(monthVal % 100);
		elseif (monthVal < 10000) then
			monthVal = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((monthVal / 100) % 100, monthVal % 100)
		else
			monthVal = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(monthVal / 100 / 100, (monthVal / 100) % 100, monthVal % 100)
		end
	end
	
	if (yearVal == 0) then
		yearVal = "|cFF8888880|r";
	else
		if (yearVal < 100) then
			yearVal = ("|cFFFF6600%dc |r"):format(yearVal % 100);
		elseif (yearVal < 10000) then
			yearVal = ("|cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format((yearVal / 100) % 100, yearVal % 100)
		else
			yearVal = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(yearVal / 100 / 100, (yearVal / 100) % 100, yearVal % 100)
		end
	end
	
	if (method == "world") then
		GTP.W_D_Frame:SetText(dayVal);
		GTP.W_W_Frame:SetText(weekVal);
		GTP.W_M_Frame:SetText(monthVal);
		GTP.W_Y_Frame:SetText(yearVal);
	elseif (method == "instance") then
		GTP.I_D_Frame:SetText(dayVal);
		GTP.I_W_Frame:SetText(weekVal);
		GTP.I_M_Frame:SetText(monthVal);
		GTP.I_Y_Frame:SetText(yearVal);
	elseif (method == "quest") then
		GTP.Q_D_Frame:SetText(dayVal);
		GTP.Q_W_Frame:SetText(weekVal);
		GTP.Q_M_Frame:SetText(monthVal);
		GTP.Q_Y_Frame:SetText(yearVal);
	elseif (method == "vendor") then
		GTP.V_D_Frame:SetText(dayVal);
		GTP.V_W_Frame:SetText(weekVal);
		GTP.V_M_Frame:SetText(monthVal);
		GTP.V_Y_Frame:SetText(yearVal);
	elseif (method == "spent") then
		GTP.S_D_Frame:SetText(dayVal);
		GTP.S_W_Frame:SetText(weekVal);
		GTP.S_M_Frame:SetText(monthVal);
		GTP.S_Y_Frame:SetText(yearVal);
	elseif (method == "repair") then
		GTP.R_D_Frame:SetText(dayVal);
		GTP.R_W_Frame:SetText(weekVal);
		GTP.R_M_Frame:SetText(monthVal);
		GTP.R_Y_Frame:SetText(yearVal);
	end
end

local function InitWindowLocations(self,point,xLoc,yLoc)
	self:ClearAllPoints();
	if (point == 'nil') then
		point = "CENTER";
	end
	
	self:SetPoint(point,xLoc,yLoc);
end	

local function SavedVarsConstructor(data,aux)
	if (data == "quest" or data == "vendor" or data == "spent" or data == "repair") then
		if (type(GTP_db[plName][data]) ~= "table") then
			GTP_db[plName][data] = {
				[curDay] = 0,
				week = 0,
				[curMonth] = 0,
				[curYear] = 0,
				['all-time'] = 0,
			};
		else
			GTP_db[plName][data][aux] = 0;
		end
	elseif (data == "mail") then
		if (type(GTP_db[plName].mail) ~= "table") then
			GTP_db[plName].mail = {
				cod = {
					inc = {
						[curDay] = 0,
						week = 0,
						[curMonth] = 0,
						[curYear] = 0,
						['all-time'] = 0,
					},
					out = {
						[curDay] = 0,
						week = 0,
						[curMonth] = 0,
						[curYear] = 0,
						['all-time'] = 0,
					},
					pending = {},
					history = {
						inc = {},
						out = {},
					},
				},
				postage = {
					[curDay] = 0,
					week = 0,
					[curMonth] = 0,
					[curYear] = 0,
					['all-time'] = 0,
				},
				gold = {
					inc = {
						[curDay] = 0,
						week = 0,
						[curMonth] = 0,
						[curYear] = 0,
						['all-time'] = 0,
					},
					out = {
						[curDay] = 0,
						week = 0,
						[curMonth] = 0,
						[curYear] = 0,
						['all-time'] = 0,
					},
					history = {
						inc = {},
						out = {},
					},
				}
			};
		else
			GTP_db[plName].mail.cod.inc[aux] = 0;
			GTP_db[plName].mail.cod.out[aux] = 0;
			GTP_db[plName].mail.postage[aux] = 0;
			GTP_db[plName].mail.gold.inc[aux] = 0;
			GTP_db[plName].mail.gold.out[aux] = 0;
		end
	else

	-- Re-Initialize Instance Variables
		GTP_db[plName].instance[data] = {
			total = 0,
			solo = 0,
			group = 0,
			raid = 0,
		}
		
		-- Re-Initialize World Variables
		GTP_db[plName].world[data] = {
			total = 0,
			solo = 0,
			group = 0,
			raid = 0,
		}
	end
end;

local function CheckTimeResets()
	-- Reset Day variables for a new day
	if (GTP_db[plName].curDay ~= curDay) then
		local oldDay = GTP_db[plName].curDay;
		GTP_db[plName].curDay = curDay;
		
		-- Reset day variables
		GTP_db[plName].instance[oldDay] = nil;
		GTP_db[plName].world[oldDay] = nil;
		GTP_db[plName].quest[oldDay] = nil;
		GTP_db[plName].vendor[oldDay] = nil;
		GTP_db[plName].spent[oldDay] = nil;
		GTP_db[plName].repair[oldDay] = nil;
		
		
		GTP_db[plName].mail.cod.inc[oldDay] = nil;
		GTP_db[plName].mail.cod.out[oldDay] = nil;
		GTP_db[plName].mail.postage[oldDay] = nil;
		GTP_db[plName].mail.gold.inc[oldDay] = nil;
		GTP_db[plName].mail.gold.out[oldDay] = nil;
		
		SavedVarsConstructor(curDay,'');
		SavedVarsConstructor('quest',curDay);
		SavedVarsConstructor('vendor',curDay);
		SavedVarsConstructor('spent',curDay);
		SavedVarsConstructor('repair',curDay);
		SavedVarsConstructor('mail',curDay);
	end
	
	-- Reset Month variables for a new month
	if (GTP_db[plName].curMonth ~= curMonth) then
		local oldMonth = GTP_db[plName].curMonth;
		GTP_db[plName].curMonth = curMonth;
		
		-- Reset day variables
		GTP_db[plName].instance[oldMonth] = nil;
		GTP_db[plName].world[oldMonth] = nil;
		GTP_db[plName].quest[oldMonth] = nil;
		GTP_db[plName].vendor[oldMonth] = nil;
		GTP_db[plName].spent[oldMonth] = nil;
		GTP_db[plName].repair[oldMonth] = nil;
		
		
		GTP_db[plName].mail.cod.inc[curMonth] = nil;
		GTP_db[plName].mail.cod.out[curMonth] = nil;
		GTP_db[plName].mail.postage[curMonth] = nil;
		GTP_db[plName].mail.gold.inc[curMonth] = nil;
		GTP_db[plName].mail.gold.out[curMonth] = nil;
		
		GTP_db[plName].archive[curYear][curMonth] = {
			instance = 0,
			world = 0,
			vendor = 0,
			spent = 0,
			repair = 0,
			quest = 0,
			mail = {
				cod = {
					inc = 0,
					out = 0,
				},
				postage = 0,
				gold = {
					inc = 0,
					out = 0,
				}
			}
		}
		
		SavedVarsConstructor(curMonth,'');
		SavedVarsConstructor('quest',curMonth);
		SavedVarsConstructor('vendor',curMonth);
		SavedVarsConstructor('spent',curMonth);
		SavedVarsConstructor('repair',curMonth);
		SavedVarsConstructor('mail',curMonth);
	end
	
	-- Reset Year variables for a new year
	if (GTP_db[plName].curYear ~= curYear) then
		local oldYear = GTP_db[plName].curYear;
		GTP_db[plName].curYear = curYear;
		
		-- Reset day variables
		GTP_db[plName].instance[oldYear] = nil;
		GTP_db[plName].world[oldYear] = nil;
		GTP_db[plName].quest[oldYear] = nil;
		GTP_db[plName].vendor[oldYear] = nil;
		GTP_db[plName].spent[oldYear] = nil;
		GTP_db[plName].repair[oldYear] = nil;
		
		
		GTP_db[plName].mail.cod.inc[curYear] = nil;
		GTP_db[plName].mail.cod.out[curYear] = nil;
		GTP_db[plName].mail.postage[curYear] = nil;
		GTP_db[plName].mail.gold.inc[curYear] = nil;
		GTP_db[plName].mail.gold.out[curYear] = nil;
		
		GTP_db[plName].archive[curYear] = {
			instance = 0,
			world = 0,
			vendor = 0,
			spent = 0,
			repair = 0,
			quest = 0,
			mail = {
				cod = {
					inc = 0,
					out = 0,
				},
				postage = 0,
				gold = {
					inc = 0,
					out = 0,
				}
			}
		}
		
		SavedVarsConstructor(curYear,'');
		SavedVarsConstructor('quest',curYear);
		SavedVarsConstructor('vendor',curYear);
		SavedVarsConstructor('spent',curYear);
		SavedVarsConstructor('repair',curYear);
		SavedVarsConstructor('mail',curYear);
	end
	
	if (curDay == "Sunday" and isWeekA and not weekChange) then
		isWeekA = false;
		isWeekB = true;
		weekChange = true;
		SavedVarsConstructor("week",'');
		SavedVarsConstructor("quest",'week');
		SavedVarsConstructor("vendor",'week');
		SavedVarsConstructor("spent",'week');
		SavedVarsConstructor("repair",'week');
		SavedVarsConstructor("mail",'week');
	elseif (curDay == "Sunday" and isWeekB and not weekChange) then
		isWeekA = true;
		isWeekB = false;
		weekChange = true;
		SavedVarsConstructor("week");
		SavedVarsConstructor("quest",'week');
		SavedVarsConstructor("vendor",'week');
		SavedVarsConstructor("spent",'week');
		SavedVarsConstructor("repair",'week');
		SavedVarsConstructor("mail",'week');
	end
	
	if (curDay == "Monday" and weekChange) then
		weekChange = false;
	end
	
end

local function InitializeSavedVars()
	GTP_db[plName] = {
		['all-time'] = 0,
		['curDay'] = curDay,
		['curMonth'] = curMonth,
		['curYear'] = curYear,
		instance = {
			[curDay] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			week = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			[curMonth] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			[curYear] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			['all-time'] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
		},
		world = {
			[curDay] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			week = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			[curMonth] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			[curYear] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
			['all-time'] = {
				total = 0,
				solo = 0,
				group = 0,
				raid = 0,
			},
		},
		archive = {
			years = curYear,
			[curYear] = {
				instance = 0,
				world = 0,
				vendor = 0,
				spent = 0,
				repair = 0,
				quest = 0,
				mail = {
					cod = {
						inc = 0,
						out = 0
					},
					postage = 0,
					gold = {
						inc = 0,
						out = 0
					}
				},
				[curMonth] = {
					instance = 0,
					world = 0,
					vendor = 0,
					spent = 0,
					repair = 0,
					quest = 0,
					mail = {
						cod = {
							inc = 0,
							out = 0
						},
						postage = 0,
						gold = {
							inc = 0,
							out = 0
						}
					},
				}
				
			}
		},
		WinLoc = {
			Main = {
				A = "CENTER",
				X = 0,
				Y = 0,
			},
			Archive = {
				A = "CENTER",
				X = 380,
				Y = 99,
			}
		}
	}
end

local function InitAddon()
	table.insert(itr,tostring(iCtr)..". InitAddon()");
	
	weekday,month,day,year = CalendarGetDate();
	table.insert(itr,"----- Day: '"..day.."'");
	table.insert(itr,"----- Weekday: '"..weekday.."'");
	table.insert(itr,"----- Month: '"..month.."'");
	table.insert(itr,"----- Year: '"..year.."'");
	
	--print("Month: '"..tostring(month).."'");
	curDay = weekdays[weekday];
	table.insert(itr,"----- CurDay: '"..curDay.."' ("..weekday..")");
	curMonth = months[month];
	table.insert(itr,"----- CurMonth: '"..curMonth.."' ("..month..")");
	curYear = tostring(year);
	table.insert(itr,"----- CurYear: '"..curYear.."' ("..year..")");
	table.insert(itr,"----- Player: '"..tostring(plName).."'");
	iCtr = (iCtr or 0) + 1;
	
	if (type(GTP_db) ~= "table") then
		print("Creating GTP_db");
		GTP_db = {};
	end
	if (type(GTP_db[plName]) ~= "table") then
		print("GTP_db[plName] not a table");
		InitializeSavedVars()
	
		SavedVarsConstructor(curYear,'');
		SavedVarsConstructor(curMonth,'');
		SavedVarsConstructor("week",'');
		SavedVarsConstructor(curDay,'');
		SavedVarsConstructor("quest",'');
		SavedVarsConstructor("vendor",'');
		SavedVarsConstructor("spent",'');
		SavedVarsConstructor("repair",'');
		SavedVarsConstructor("mail",'');
	end
	
	-- BETA-r14: Append window location variables
	if (type(GTP_db[plName].WinLoc) ~= "table") then
		GTP_db[plName].WinLoc = {
			Main = {
				A = "CENTER",
				X = 0,
				Y = 0,
			},
			Archive = {
				A = "CENTER",
				X = 380,
				Y = 99,
			}
		}
	end
	
	-- BETA-r14: Append mail variables
	if (type(GTP_db[plName].mail) ~= "table") then
		GTP_db[plName].mail = {
			cod = {
				inc = {
					[curDay] = 0,
					week = 0,
					[curMonth] = 0,
					[curYear] = 0,
					['all-time'] = 0,
				},
				out = {
					[curDay] = 0,
					week = 0,
					[curMonth] = 0,
					[curYear] = 0,
					['all-time'] = 0,
				},
				pending = {},
				history = {
					inc = {},
					out = {},
				},
			},
			postage = {
				[curDay] = 0,
				week = 0,
				[curMonth] = 0,
				[curYear] = 0,
				['all-time'] = 0,
			},
			gold = {
				inc = {
					[curDay] = 0,
					week = 0,
					[curMonth] = 0,
					[curYear] = 0,
					['all-time'] = 0,
				},
				out = {
					[curDay] = 0,
					week = 0,
					[curMonth] = 0,
					[curYear] = 0,
					['all-time'] = 0,
				},
				history = {
					inc = {},
					out = {},
				},
			}
		};
		
		GTP_db[plName].archive[curYear] = {
			mail = {
				cod = {
					inc = 0,
					out = 0,
				},
				postage = 0,
				gold = {
					inc = 0,
					out = 0,
				}
			}
		}
		
		GTP_db[plName].archive[curYear][curMonth] = {
			mail = {
				cod = {
					inc = 0,
					out = 0,
				},
				postage = 0,
				gold = {
					inc = 0,
					out = 0,
				}
			}
		}
	end
	
	-- Check to see if any timeframes need to be reset
	CheckTimeResets();
	
	-- Set window locations from saved layout data
	InitWindowLocations(GoldDialog,GTP_db[plName].WinLoc.Main.A,GTP_db[plName].WinLoc.Main.X,GTP_db[plName].WinLoc.Main.Y);
	InitWindowLocations(ArchiveDialog,GTP_db[plName].WinLoc.Archive.A,GTP_db[plName].WinLoc.Archive.X,GTP_db[plName].WinLoc.Archive.Y);
	
	-- INITIALIZE FRAMES --
	GTP.initGoldDialog(GoldDialog);
	GTP.initArchiveDialog(ArchiveDialog);
	
	FormatValues(GTP_db[plName].world[curDay].total,GTP_db[plName].world.week.total,GTP_db[plName].world[curMonth].total,GTP_db[plName].world[curYear].total,'',"world");
	FormatValues(GTP_db[plName].instance[curDay].total,GTP_db[plName].instance.week.total,GTP_db[plName].instance[curMonth].total,GTP_db[plName].instance[curYear].total,'',"instance");
	FormatValues(GTP_db[plName].quest[curDay],GTP_db[plName].quest.week,GTP_db[plName].quest[curMonth],GTP_db[plName].quest[curYear],'',"quest");
	FormatValues(GTP_db[plName].vendor[curDay],GTP_db[plName].vendor.week,GTP_db[plName].vendor[curMonth],GTP_db[plName].vendor[curYear],'',"vendor");
	FormatValues(GTP_db[plName].spent[curDay],GTP_db[plName].spent.week,GTP_db[plName].spent[curMonth],GTP_db[plName].spent[curYear],'',"spent");
	FormatValues(GTP_db[plName].repair[curDay],GTP_db[plName].repair.week,GTP_db[plName].repair[curMonth],GTP_db[plName].repair[curYear],'',"repair");
	-- MAIL VALUES
	-- COD VALUES
	FormatAllTimeValues(GTP_db[plName].world["all-time"].total,GTP_db[plName].instance["all-time"].total,GTP_db[plName].quest["all-time"],GTP_db[plName].vendor["all-time"],GTP_db[plName].spent["all-time"],GTP_db[plName].repair["all-time"]);
end;



local function GetMoneyString()
	local copper = GetMoney();
	return ("%d%d%d"):format(copper % 100, (copper / 100) % 100, copper / 100 / 100);
end;

local function CurrentMoney()
	
	curMoney = GetMoney();
end;

local function GetMoneyDifference()
	local moneyDiff = 0;
	
	--print("Cur: "..curMoney);
	NewMoney = GetMoney();
	--print("New: "..NewMoney);
	moneyDiff = NewMoney - curMoney;
	--print("Mod: "..moneyDiff);
	curMoney = GetMoney();
	--print("---------");
	--print("Cur: "..curMoney);
	return moneyDiff;
end

local function MerchantMoney()
	local moneyDiff;
	moneyDiff = GetMoneyDifference();
	
	if (isPurchase) then
		moneyDiff = moneyDiff * -1;
		SpendTotal = (SpendTotal or 0) + moneyDiff;
		isPurchase = false;
	elseif (isRepair) then
		moneyDiff = moneyDiff * -1;
		RepairTotal = (RepairTotal or 0) + moneyDiff;
		isRepair = false;
	else
		if (NewMoney > curMoney) then
			GainTotal = (GainTotal or 0) - moneyDiff;
		else
			GainTotal = (GainTotal or 0) + moneyDiff;
		end
	end
	DebugPrint("Spent: '"..tostring(SpendTotal).."'","merchant");
	DebugPrint("Gained: '"..tostring(GainTotal).."'","merchant");
	DebugPrint("Repaired: '"..tostring(RepairTotal).."'","merchant");
end

local function UpdateMoney(isCloseMerchant)
	local moneyDiff = 0;
	local questMoney = 0;
	if (isQuest) then
		questMoney = GetRewardMoney();
	else
		moneyDiff = GetMoneyDifference();
	end
	--print("Money Difference 2: "..tostring(moneyDiff));
	--local totalMoney = ("%d%d%d"):format(moneyDiff / 100 / 100, (moneyDiff / 100) % 100, moneyDiff % 100);
	if (isQuest) then
		GTP_db[plName].quest[curDay] = GTP_db[plName].quest[curDay] + questMoney;
		GTP_db[plName].quest[curMonth] = GTP_db[plName].quest[curMonth] + questMoney;
		GTP_db[plName].quest.week = GTP_db[plName].quest.week + questMoney;
		GTP_db[plName].quest[curYear] = GTP_db[plName].quest[curYear] + questMoney;
		GTP_db[plName].quest["all-time"] = GTP_db[plName].quest["all-time"] + questMoney;
		GTP_db[plName]["all-time"] = GTP_db[plName]["all-time"] + questMoney;
		GTP_db[plName].archive[curYear].quest = GTP_db[plName].archive[curYear].quest + questMoney;
		GTP_db[plName].archive[curYear][curMonth].quest = GTP_db[plName].archive[curYear][curMonth].quest + questMoney;
	elseif (isCloseMerchant) then
		DebugPrint("Not purchase","merchant");
		if ((GainTotal or 0) > 0) then
			GTP_db[plName].vendor[curDay] = GTP_db[plName].vendor[curDay] + GainTotal;
			GTP_db[plName].vendor[curMonth] = GTP_db[plName].vendor[curMonth] + GainTotal;
			GTP_db[plName].vendor.week = GTP_db[plName].vendor.week + GainTotal;
			GTP_db[plName].vendor[curYear] = GTP_db[plName].vendor[curYear] + GainTotal;
			GTP_db[plName].vendor["all-time"] = GTP_db[plName].vendor["all-time"] + GainTotal;
			GTP_db[plName]["all-time"] = GTP_db[plName]["all-time"] + GainTotal;
			GTP_db[plName].archive[curYear].vendor = GTP_db[plName].archive[curYear].vendor + GainTotal;
			GTP_db[plName].archive[curYear][curMonth].vendor = GTP_db[plName].archive[curYear][curMonth].vendor + GainTotal;
		end
		if ((SpendTotal or 0) > 0) then
			GTP_db[plName].spent[curDay] = GTP_db[plName].spent[curDay] + SpendTotal;
			GTP_db[plName].spent[curMonth] = GTP_db[plName].spent[curMonth] + SpendTotal;
			GTP_db[plName].spent.week = GTP_db[plName].spent.week + SpendTotal;
			GTP_db[plName].spent[curYear] = GTP_db[plName].spent[curYear] + SpendTotal;
			GTP_db[plName].spent["all-time"] = GTP_db[plName].spent["all-time"] + SpendTotal;
			GTP_db[plName].archive[curYear].spent = GTP_db[plName].archive[curYear].spent + SpendTotal;
			GTP_db[plName].archive[curYear][curMonth].spent = GTP_db[plName].archive[curYear][curMonth].spent + SpendTotal;
		end
		if ((RepairTotal or 0) > 0) then
			GTP_db[plName].repair[curDay] = GTP_db[plName].repair[curDay] + RepairTotal;
			GTP_db[plName].repair[curMonth] = GTP_db[plName].repair[curMonth] + RepairTotal;
			GTP_db[plName].repair.week = GTP_db[plName].repair.week + RepairTotal;
			GTP_db[plName].repair[curYear] = GTP_db[plName].repair[curYear] + RepairTotal;
			GTP_db[plName].repair["all-time"] = GTP_db[plName].repair["all-time"] + RepairTotal;
			GTP_db[plName].archive[curYear].repair = GTP_db[plName].archive[curYear].repair + RepairTotal;
			GTP_db[plName].archive[curYear][curMonth].repair = GTP_db[plName].archive[curYear][curMonth].repair + RepairTotal;
		end
		GainTotal = 0;
		SpendTotal = 0;
		RepairTotal = 0;
	elseif (mailVars.isMail) then
		if (mailVars.mailType.sending) then
			--print("Mail sending");
			local moneySum = (mailVars.money.gold * 10000) + (mailVars.money.silver * 100) + mailVars.money.copper;
			if (mailVars.numItems > 0) then
				--print("Sending Items");
				moneyDiff = moneyDiff * -1;
				
				if (mailVars.isCOD) then
					--print("Is C.O.D. for: "..tostring(mailVars.money.gold).."g "..tostring(mailVars.money.silver).."s "..tostring(mailVars.money.copper).."c");
					GTP_db[plName].mail.cod.pending[mailVars.mailName] = moneySum;
				elseif (mailVars.isSendMoney and moneySum > 0) then
					--print("Is Sending Money of : "..tostring(mailVars.money.gold).."g "..tostring(mailVars.money.silver).."s "..tostring(mailVars.money.copper).."c");
					
					if (type(GTP_db[plName].mail.gold.history.out[mailVars.mailName]) ~= "table") then
						GTP_db[plName].mail.gold.history.out[mailVars.mailName] = {
							[month.."-"..day.."-"..year] = moneySum;
						};
					else
						GTP_db[plName].mail.gold.history.out[mailVars.mailName][month.."-"..day.."-"..year] = moneySum;
					end
					GTP_db[plName].mail.gold.out[curDay] = GTP_db[plName].mail.gold.out[curDay] + moneySum;
					GTP_db[plName].mail.gold.out[curMonth] = GTP_db[plName].mail.gold.out[curMonth] + moneySum;
					GTP_db[plName].mail.gold.out.week = GTP_db[plName].mail.gold.out.week + moneySum;
					GTP_db[plName].mail.gold.out[curYear] = GTP_db[plName].mail.gold.out[curYear] + moneySum;
					GTP_db[plName].mail.gold.out["all-time"] = GTP_db[plName].mail.gold.out["all-time"] + moneySum;
					GTP_db[plName].archive[curYear].mail.gold.out = GTP_db[plName].archive[curYear].mail.gold.out + moneySum;
					GTP_db[plName].archive[curYear][curMonth].mail.gold.out = GTP_db[plName].archive[curYear][curMonth].mail.gold.out + moneySum;
				end
			elseif (mailVars.isSendMoney and moneySum > 0) then
				--print("Is Sending Money of : "..tostring(mailVars.money.gold).."g "..tostring(mailVars.money.silver).."s "..tostring(mailVars.money.copper).."c");
				
				if (type(GTP_db[plName].mail.gold.history.out[mailVars.mailName]) ~= "table") then
					GTP_db[plName].mail.gold.history.out[mailVars.mailName] = {
						[month.."-"..day.."-"..year] = moneySum;
					};
				else
					GTP_db[plName].mail.gold.history.out[mailVars.mailName][month.."-"..day.."-"..year] = moneySum;
				end
				GTP_db[plName].mail.gold.out[curDay] = GTP_db[plName].mail.gold.out[curDay] + moneySum;
				GTP_db[plName].mail.gold.out[curMonth] = GTP_db[plName].mail.gold.out[curMonth] + moneySum;
				GTP_db[plName].mail.gold.out.week = GTP_db[plName].mail.gold.out.week + moneySum;
				GTP_db[plName].mail.gold.out[curYear] = GTP_db[plName].mail.gold.out[curYear] + moneySum;
				GTP_db[plName].mail.gold.out["all-time"] = GTP_db[plName].mail.gold.out["all-time"] + moneySum;
				GTP_db[plName].archive[curYear].mail.gold.out = GTP_db[plName].archive[curYear].mail.gold.out + moneySum;
				GTP_db[plName].archive[curYear][curMonth].mail.gold.out = GTP_db[plName].archive[curYear][curMonth].mail.gold.out + moneySum;
			end
			
			--print("Postage: "..tostring(mailVars.postage));
			GTP_db[plName].mail.postage[curDay] = GTP_db[plName].mail.postage[curDay] + mailVars.postage;
			GTP_db[plName].mail.postage[curMonth] = GTP_db[plName].mail.postage[curMonth] + mailVars.postage;
			GTP_db[plName].mail.postage.week = GTP_db[plName].mail.postage.week + mailVars.postage;
			GTP_db[plName].mail.postage[curYear] = GTP_db[plName].mail.postage[curYear] + mailVars.postage;
			GTP_db[plName].mail.postage["all-time"] = GTP_db[plName].mail.postage["all-time"] + mailVars.postage;
			GTP_db[plName].archive[curYear].mail.postage = GTP_db[plName].archive[curYear].mail.postage + mailVars.postage;
			GTP_db[plName].archive[curYear][curMonth].mail.postage = GTP_db[plName].archive[curYear][curMonth].mail.postage + mailVars.postage;
		elseif (mailVars.mailType.receiving) then
			--print("Receiving Mail");
			if (moneyDiff > 0) then
				if (GTP_db[plName].mail.cod.pending[mailVars.mailName] and moneyDiff == GTP_db[plName].mail.cod.pending[mailVars.mailName] and mailVars.isCOD) then
					--print("Receiving C.O.D. payment");
					if (type(GTP_db[plName].mail.cod.history.inc[mailVars.mailName]) ~= "table") then
						GTP_db[plName].mail.cod.history.inc[mailVars.mailName] = {
							[month.."-"..day.."-"..year] = moneyDiff;
						};
					else
						GTP_db[plName].mail.cod.history.inc[mailVars.mailName][month.."-"..day.."-"..year] = moneyDiff;
					end
					GTP_db[plName].mail.cod.pending[mailVars.mailName] = nil;
					GTP_db[plName].mail.cod.inc[curDay] = GTP_db[plName].mail.cod.inc[curDay] + moneyDiff;
					GTP_db[plName].mail.cod.inc[curMonth] = GTP_db[plName].mail.cod.inc[curMonth] + moneyDiff;
					GTP_db[plName].mail.cod.inc.week = GTP_db[plName].mail.cod.inc.week + moneyDiff;
					GTP_db[plName].mail.cod.inc[curYear] = GTP_db[plName].mail.cod.inc[curYear] + moneyDiff;
					GTP_db[plName].mail.cod.inc["all-time"] = GTP_db[plName].mail.cod.inc["all-time"] + moneyDiff;
					GTP_db[plName].archive[curYear].mail.cod.inc = GTP_db[plName].archive[curYear].mail.cod.inc + moneyDiff;
					GTP_db[plName].archive[curYear][curMonth].mail.cod.inc = GTP_db[plName].archive[curYear][curMonth].mail.cod.inc + moneyDiff;
				else
					--print("Receiving Money");
					if (type(GTP_db[plName].mail.gold.history.inc[mailVars.mailName]) ~= "table") then
						GTP_db[plName].mail.gold.history.inc[mailVars.mailName] = {
							[month.."-"..day.."-"..year] = moneyDiff;
						};
					else
						GTP_db[plName].mail.gold.history.inc[mailVars.mailName][month.."-"..day.."-"..year] = moneyDiff;
					end
					GTP_db[plName].mail.gold.inc[curDay] = GTP_db[plName].mail.gold.inc[curDay] + moneyDiff;
					GTP_db[plName].mail.gold.inc[curMonth] = GTP_db[plName].mail.gold.inc[curMonth] + moneyDiff;
					GTP_db[plName].mail.gold.inc.week = GTP_db[plName].mail.gold.inc.week + moneyDiff;
					GTP_db[plName].mail.gold.inc[curYear] = GTP_db[plName].mail.gold.inc[curYear] + moneyDiff;
					GTP_db[plName].mail.gold.inc["all-time"] = GTP_db[plName].mail.gold.inc["all-time"] + moneyDiff;
					GTP_db[plName].archive[curYear].mail.gold.inc = GTP_db[plName].archive[curYear].mail.gold.inc + moneyDiff;
					GTP_db[plName].archive[curYear][curMonth].mail.gold.inc = GTP_db[plName].archive[curYear][curMonth].mail.gold.inc + moneyDiff;
				end
			elseif (moneyDiff < 0) then
				--print("Paying for C.O.D. item");
				moneyDiff = moneyDiff * -1;
				if (type(GTP_db[plName].mail.cod.history.out[mailVars.mailName]) ~= "table") then
					GTP_db[plName].mail.cod.history.out[mailVars.mailName] = {
						[month.."-"..day.."-"..year] = moneyDiff;
					};
				else
					GTP_db[plName].mail.cod.history.out[mailVars.mailName][month.."-"..day.."-"..year] = moneyDiff;
				end
				GTP_db[plName].mail.cod.out[curDay] = GTP_db[plName].mail.cod.out[curDay] + moneyDiff;
				GTP_db[plName].mail.cod.out[curMonth] = GTP_db[plName].mail.cod.out[curMonth] + moneyDiff;
				GTP_db[plName].mail.cod.out.week = GTP_db[plName].mail.cod.out.week + moneyDiff;
				GTP_db[plName].mail.cod.out[curYear] = GTP_db[plName].mail.cod.out[curYear] + moneyDiff;
				GTP_db[plName].mail.cod.out["all-time"] = GTP_db[plName].mail.cod.out["all-time"] + moneyDiff;
				GTP_db[plName].archive[curYear].mail.cod.out = GTP_db[plName].archive[curYear].mail.cod.out + moneyDiff;
				GTP_db[plName].archive[curYear][curMonth].mail.cod.out = GTP_db[plName].archive[curYear][curMonth].mail.cod.out + moneyDiff;
			end
		end
		
		-- Reset all mail variables
		mailVars = {
			isMail = false,
			isPostage = false,
			isGoldCheck = false,
			mailType = {
				receiving = mailVars.mailType.receiving,
				sending = mailVars.mailType.sending,
			},
			isSendMoney = false,
			isCOD = false,
			numItems = 0,
			mailName = '',
			postage = 0,
			money = {
				gold = 0,
				silver = 0,
				copper = 0,
			}
		}
	elseif (isInstance) then
		DebugPrint("INSTANCE_MONEY","instance");
		-- Instance Day Adjust
		GTP_db[plName].instance[curDay].total = GTP_db[plName].instance[curDay].total + moneyDiff;
		GTP_db[plName].instance[curDay][trackType] = GTP_db[plName].instance[curDay][trackType] + moneyDiff;
		-- Instance Week Adjust
		GTP_db[plName].instance.week.total = GTP_db[plName].instance.week.total + moneyDiff;
		GTP_db[plName].instance.week[trackType] = GTP_db[plName].instance.week[trackType] + moneyDiff;
		-- Instance Month Adjust
		GTP_db[plName].instance[curMonth].total = GTP_db[plName].instance[curMonth].total + moneyDiff;
		GTP_db[plName].instance[curMonth][trackType] = GTP_db[plName].instance[curMonth][trackType] + moneyDiff;
		-- Instance Year Adjust
		GTP_db[plName].instance[curYear].total = GTP_db[plName].instance[curYear].total + moneyDiff;
		GTP_db[plName].instance[curYear][trackType] = GTP_db[plName].instance[curYear][trackType] + moneyDiff;
		-- Instance All-Time Adjust
		GTP_db[plName].instance["all-time"].total = GTP_db[plName].instance["all-time"].total + moneyDiff;
		GTP_db[plName].instance["all-time"][trackType] = GTP_db[plName].instance["all-time"][trackType] + moneyDiff;
		GTP_db[plName]["all-time"] = GTP_db[plName]["all-time"] + moneyDiff;
		GTP_db[plName].archive[curYear].instance = GTP_db[plName].archive[curYear].instance + moneyDiff;
		GTP_db[plName].archive[curYear][curMonth].instance = GTP_db[plName].archive[curYear][curMonth].instance + moneyDiff;
	elseif (not isInstance) then
		DebugPrint("WORLD_MONEY .."..tostring(moneyDiff),"world");
		-- World Day Adjust
		GTP_db[plName].world[curDay].total = GTP_db[plName].world[curDay].total + moneyDiff;
		GTP_db[plName].world[curDay][trackType] = GTP_db[plName].world[curDay][trackType] + moneyDiff;
		-- World Week Adjust
		GTP_db[plName].world.week.total = GTP_db[plName].world.week.total + moneyDiff;
		GTP_db[plName].world.week[trackType] = GTP_db[plName].world.week[trackType] + moneyDiff;
		-- World Month Adjust
		GTP_db[plName].world[curMonth].total = GTP_db[plName].world[curMonth].total + moneyDiff;
		GTP_db[plName].world[curMonth][trackType] = GTP_db[plName].world[curMonth][trackType] + moneyDiff;
		-- World Year Adjust
		GTP_db[plName].world[curYear].total = GTP_db[plName].world[curYear].total + moneyDiff;
		GTP_db[plName].world[curYear][trackType] = GTP_db[plName].world[curYear][trackType] + moneyDiff;
		-- World All-Time Adjust
		GTP_db[plName].world["all-time"].total = GTP_db[plName].world["all-time"].total + moneyDiff;
		GTP_db[plName].world["all-time"][trackType] = GTP_db[plName].world["all-time"][trackType] + moneyDiff;
		GTP_db[plName]["all-time"] = GTP_db[plName]["all-time"] + moneyDiff;
		GTP_db[plName].archive[curYear].world = GTP_db[plName].archive[curYear].world + moneyDiff;
		GTP_db[plName].archive[curYear][curMonth].world = GTP_db[plName].archive[curYear][curMonth].world + moneyDiff;
	end

	
	FormatValues(GTP_db[plName].world[curDay].total,GTP_db[plName].world.week.total,GTP_db[plName].world[curMonth].total,GTP_db[plName].world[curYear].total,'','world');
	FormatValues(GTP_db[plName].instance[curDay].total,GTP_db[plName].instance.week.total,GTP_db[plName].instance[curMonth].total,GTP_db[plName].instance[curYear].total,'',"instance")
	FormatValues(GTP_db[plName].quest[curDay],GTP_db[plName].quest.week,GTP_db[plName].quest[curMonth],GTP_db[plName].quest[curYear],'','quest');
	FormatValues(GTP_db[plName].vendor[curDay],GTP_db[plName].vendor.week,GTP_db[plName].vendor[curMonth],GTP_db[plName].vendor[curYear],'',"vendor")
	FormatValues(GTP_db[plName].spent[curDay],GTP_db[plName].spent.week,GTP_db[plName].spent[curMonth],GTP_db[plName].spent[curYear],'',"spent")
	FormatValues(GTP_db[plName].repair[curDay],GTP_db[plName].repair.week,GTP_db[plName].repair[curMonth],GTP_db[plName].repair[curYear],'',"repair")
	FormatAllTimeValues(GTP_db[plName].world["all-time"].total,GTP_db[plName].instance["all-time"].total,GTP_db[plName].quest["all-time"],GTP_db[plName].vendor["all-time"],GTP_db[plName].spent["all-time"],GTP_db[plName].repair["all-time"]);
	
	--print("Final CurMoney");
	curMoney = GetMoney();
end;

local function GetTrackType()
	--[[if (GetNumGroupMembers() == 0) then
		trackType = 'solo';
	elseif (GetNumGroupMembers() > 0 and GetNumGroupMembers() < 5) then
		trackType = 'group';
	elseif (GetNumGroupMembers() > 5) then
		trackType = 'raid';
	end]]
	if (IsInRaid()) then
		trackType = 'raid';
	elseif (IsInGroup()) then
		trackType = 'group';
	else
		trackType = 'solo';
	end
	
	if (IsInInstance()) then
		isInstance = true;
		isTracking = true;
	else
		isInstance = false;
		isTracking = false;
	end
end

-- MAIL: Check to see if player receiving mail
MailFrameTab1:HookScript("OnClick",function(self)
	mailVars.mailType.receiving = true;
	mailVars.mailType.sending = false;
end);

-- MAIL: Check to see if player sending mail
MailFrameTab2:HookScript("OnClick",function(self)
	mailVars.mailType.sending = true;
	mailVars.mailType.receiving = false;
end);

OpenMailFrame:HookScript("OnShow",function(self)
	mailVars.mailName = OpenMailSender.Name:GetText();
	if (string.find(OpenMailSubject:GetText(),"COD Payment:") > 0) then
		mailVars.isCOD = true;
	else
		mailVars.isCOD = false;
	end
end);

SendMailMailButton:HookScript("OnClick",function(self)
	mailVars.mailName = SendMailNameEditBox:GetText();
	mailVars.postage = GetSendMailPrice();
	
	if (SendMailCODButton:GetChecked()) then
		mailVars.isCOD = true;
		mailVars.isSendMoney = false;
	end
	
	if (SendMailSendMoneyButton:GetChecked()) then
		mailVars.isCOD = false;
		mailVars.isSendMoney = true;
	end
	
	for i=1,12 do
		if (GetSendMailItem(i)) then
			mailVars.numItems = mailVars.numItems + 1;
		end
	end
	
	if (SendMailMoneyGold:GetText() ~= '') then
		mailVars.money.gold = tonumber(SendMailMoneyGold:GetText());
	else
		mailVars.money.gold = 0;
	end
	
	if (SendMailMoneySilver:GetText() ~= '') then
		mailVars.money.silver = tonumber(SendMailMoneySilver:GetText());
	else
		mailVars.money.silver = 0;
	end
	
	if (SendMailMoneyCopper:GetText() ~= '') then
		mailVars.money.copper = tonumber(SendMailMoneyCopper:GetText());
	else
		mailVars.money.copper = 0;
	end
end);

StaticPopup1Button2:HookScript("OnClick",function(self)
	if (mailVars.isMail and mailVars.mailType.sending) then
		mailVars.money.gold = 0;
		mailVars.money.silver = 0;
		mailVars.money.copper = 0;
	end
end);	

Events:HookScript("OnEvent",function(self,event,...)
	if (event == "PLAYER_LOGIN") then
		--if (select(1,...) == "GoldTrackerPlus") then
			table.insert(itr,tostring(iCtr)..". ADDON_LOADED");
			iCtr = (iCtr or 0) + 1;
			InitializeVars();
			InitAddon();
		--end
	elseif (event == "PLAYER_LOGOUT") then
		local gP,_,_,gX,gY = GoldDialog:GetPoint(1);
		local aP,_,_,aX,aY = ArchiveDialog:GetPoint(1);
		
		GTP_db[plName].WinLoc.Main.A = tostring(gP);
		GTP_db[plName].WinLoc.Main.X = math.ceil(gX);
		GTP_db[plName].WinLoc.Main.Y = math.ceil(gY);
		GTP_db[plName].WinLoc.Archive = {};
		GTP_db[plName].WinLoc.Archive.A = tostring(aP);
		GTP_db[plName].WinLoc.Archive.X = math.ceil(aX);
		GTP_db[plName].WinLoc.Archive.Y = math.ceil(aY);
	elseif (event == "PLAYER_ENTERING_WORLD") then
		--[[table.insert(itr,tostring(iCtr)..". ADDON_LOADED");
		iCtr = iCtr + 1;
		InitializeVars();
		InitAddon();]]
		curMoney = GetMoney();
		--print("Day is: "..curDay);
		--print("Month is: "..curMonth);
		GetTrackType();
	elseif (event == "LOOT_OPENED") then
		--CurrentMoney();
		--Events:RegisterEvent("LOOT_CLOSED");
		--Events:RegisterEvent("CHAT_MSG_MONEY");
	elseif (event == "CHAT_MSG_MONEY") then
		UpdateMoney(false);
		--Events:UnregisterEvent("LOOT_CLOSED");
		--Events:UnregisterEvent("CHAT_MSG_MONEY");
	elseif (event == "QUEST_COMPLETE") then
		DebugPrint("QUEST COMPLETE!","quest");
		local events = GoldTrackerEvents;
		isQuest = true;
		--GTP.W_D_LabelText:SetText(tostring(isQuest));
		Events:RegisterEvent("CHAT_MSG_SYSTEM");
		Events:RegisterEvent("QUEST_FINISHED");
	elseif (event == "QUEST_FINISHED") then
		DebugPrint("QUEST FINISHED","quest");
		isQuest = false;
		--WorldDayLabelText:SetText(tostring(isQuest));
		Events:UnregisterEvent(tostring(event));
	elseif (event == "MERCHANT_SHOW") then
		DebugPrint("Merchant Open","merchant");
		local events = GoldTrackerEvents;
		isVendor = true;
		Events:RegisterEvent("ITEM_PUSH");
		Events:RegisterEvent("PLAYER_MONEY");
		Events:RegisterEvent("MERCHANT_CLOSED");
		Events:RegisterEvent("UPDATE_INVENTORY_DURABILITY");
	elseif (event == "MAIL_SHOW") then
		print("Mail opened");
		mailVars.isMail = true;
		mailVars.mailType.receiving = true;
		mailVars.mailType.sending = false;
		Events:RegisterEvent("MAIL_INBOX_UPDATE");
		Events:RegisterEvent("MAIL_SUCCESS");
		Events:RegisterEvent("MAIL_SEND_SUCCESS");
		Events:RegisterEvent("MAIL_CLOSED");
		Events:RegisterEvent("PLAYER_MONEY");
		Events:RegisterEvent("UI_INFO_MESSAGE");
		Events:RegisterEvent("ITEM_PUSH");
		
		-- TEMP Events
		Events:RegisterEvent("SEND_MAIL_COD_CHANGED");
		Events:RegisterEvent("SEND_MAIL_MONEY_CHANGED");
	elseif (event == "MAIL_INBOX_UPDATE") then
		isGoldCheck = true;
	elseif (event == "MAIL_SEND_SUCCESS") then
		isPostage = true;
	elseif (event == "MAIL_SUCCESS") then
		
		--print("Mail success");
		--print("SUC: "..curMoney);
		Events:UnregisterEvent("ITEM_UNLOCKED");
		
		
		--numItems = 0;
		--UpdateMoney(false);
	elseif (event == "MAIL_CLOSED") then
		mailVars.isMail = false;
		Events:UnregisterEvent("MAIL_SUCCESS");
		Events:UnregisterEvent("MAIL_SEND_SUCCESS");
		Events:UnregisterEvent("MAIL_CLOSED");
		Events:UnregisterEvent("PLAYER_MONEY");
		Events:UnregisterEvent("ITEM_PUSH");
		
	elseif (event == "UI_INFO_MESSAGE" and select(1,...) == "Mail sent.") then
		if (SendMailCODButton:GetChecked()) then
			Events:RegisterEvent("ITEM_UNLOCKED");
			codName = SendMailNameEditBox:GetText();
			isCOD = true;
		end
	elseif (event == "ITEM_UNLOCKED") then
		--print("ITEM UNLOCKED: '"..tostring(SendMailNameEditBox:GetText()).."'");
		--numItems = numItems + 1;
		--print(tostring(numItems).." item");
	elseif (event == "UPDATE_INVENTORY_DURABILITY") then
		isRepair = true;
	elseif (event == "PLAYER_MONEY") then
		--print("MONEY UPDATED");
		--print("Money Change");
		--print(tostring(isPurchase));
		if (isVendor or isPurchase or isRepair) then
			MerchantMoney();
		elseif (mailVars.isMail) then
			UpdateMoney(false);
		end
		--print("Old: "..tostring(curMoney));
		--print("New: "..tostring(GetMoney()));
	elseif (event == "MERCHANT_CLOSED") then
		if (isVendor) then
			isVendor = false;
			UpdateMoney(true);
			Events:UnregisterEvent(tostring(event));
			Events:UnregisterEvent("ITEM_PUSH");
			Events:UnregisterEvent("PLAYER_MONEY");
			Events:UnregisterEvent("UPDATE_INVENTORY_DURABILITY");
		end
	elseif (event == "ITEM_PUSH") then
		--print("In new items");
		if (isVendor) then
			isPurchase = true;
		elseif (mailVars.isMail and mailVars.mailType.sending and mailVars.isCOD) then
			codName = SendMailNameEditBox:GetText()

				--codName = OpenMailSender.Name:GetText();
		end
		--UpdateMoney();
	elseif (event == "CHAT_MSG_SYSTEM") then
		--[[print("IN SYSTEM");
		print("Is Quest? '"..tostring(isQuest).."'");]]
		local msg = select(1,...);
		local isQuestReward = {};
		isQuestReward[1] = string.find(msg,"Received");
		isQuestReward[2] = string.find(msg,"Gold");
		isQuestReward[3] = string.find(msg,"Silver");
		isQuestReward[4] = string.find(msg,"Copper");
		--[[print("Message: '"..msg.."'");
		print("Line #1: '"..tostring(isQuestReward[1]).."'");
		print("Line #2: '"..tostring(isQuestReward[2]).."'");
		print("Line #3: '"..tostring(isQuestReward[3]).."'");
		print("Line #4: '"..tostring(isQuestReward[4]).."'");]]
		
		if (isQuest and isQuestReward[1] and (isQuestReward[2] or isQuestReward[3] or isQuestReward[4])) then
			--print("IS QUEST");
			UpdateMoney(false);
		else
			--print("IS NOT QUEST");
		end
		--FindMoney(select(1,...));
		--print("1. "..select(1,...));
		--print("2. "..select(2,...));
		--FindMoney();
	end
	if (event == "CHAT_MSG_MONEY") then
		--print("IN MONEY!");
	end
end)

SLASH_GTRADDON1 = '/gtp';
function SlashCmdList.GTRADDON(msg, editbox)
	GoldDialog:Show();
	local command, rest = msg:match("^(%S*)%s*(.-)$");
	
	if (rest == '') then
		if (msg == '' or msg == 'help') then
			--DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFFFF  --WoW Bets--\n/wbi help - Display help information\n/wbi show - Display main game frame\n/wbi stats - Display game statistics frame");
		elseif (msg == "toggle") then
			if (GoldDialog:IsShown()) then
				GoldDialog:Hide();
				ArchiveDialog:Hide();
			else
				GoldDialog:Show();
			end
		elseif (msg == "show" or msg == "open") then
			GoldDialog:Show();
		elseif (msg == "hide" or msg == "close") then
			GoldDialog:Hide();
			ArchiveDialog:Hide();
		elseif (msg == "dump") then
			local tblSize = table.getn(itr);
			for i=1,tblSize do
				--print(itr[i]);
				print(formatter[i]);
			end
		elseif (msg == "vars") then
			print(GTP_db[plName].curDay);
		elseif (msg == "winLoc") then
			local gP,_,_,gX,gY = GoldDialog:GetPoint(1);
			local aP,_,_,aX,aY = ArchiveDialog:GetPoint(1);
			
			GTP_db[plName].WinLoc.Main.A = tostring(gP);
			GTP_db[plName].WinLoc.Main.X = math.ceil(gX);
			GTP_db[plName].WinLoc.Main.Y = math.ceil(gY);
			GTP_db[plName].WinLoc.Archive = {};
			GTP_db[plName].WinLoc.Archive.A = tostring(aP);
			GTP_db[plName].WinLoc.Archive.X = math.ceil(aX);
			GTP_db[plName].WinLoc.Archive.Y = math.ceil(aY);
			--print("Main: '"..tostring(gP).."', X: "..gX..", Y: "..gY);
			--print("Arch: '"..tostring(aP).."', X: "..aX..", Y: "..aY);
		elseif (msg == "locale") then
			for i=1,12 do
				print(L[months[i]]);
			end
		elseif (msg == "text") then
			local world = 245569;
			--local testGold = "|cFFFFFF0024g|r |cFFCCCCCC55s|r |cFFFF660069c|r"
			local testGold = ("|cFFFFFF00%dg|r |cFFCCCCCC%ds|r |cFFFF6600%dc |r"):format(world / 100 / 100, (world / 100) % 100, world % 100)
			GTP.AT_W_Frame:SetText(testGold);
		end
	else
		if (command == "debug") then
			if (rest == "time" or rest == "timeline" or rest == "t") then
				if (isDebug.timeline) then
					isDebug.timeline = false;
				else
					isDebug.timeline = true;
				end
			elseif (rest == "merch" or rest == "merchant" or rest == "m") then
				if (isDebug.merchant) then
					isDebug.merchant = false;
				else
					isDebug.merchant = true;
				end
			elseif (rest == "world" or rest == "w") then
				if (isDebug.world) then
					isDebug.world = false;
				else
					isDebug.world = true;
				end
			elseif (rest == "quest" or rest == "q") then
				if (isDebug.quest) then
					isDebug.quest = true;
				else
					isDebug.quest = false;
				end
			elseif (rest == "instance" or rest == "i") then
				if (isDebug.instance) then
					isDebug.instance = false;
				else
					isDebug.instance = true;
				end
			end
		end
	end
end